// CPSC 314 Vjan2007
// Project 2 Sample Code

//p2.cpp
//main function

//#include "stdafx.h"
#include "Vec3f.h"
#include <GL/glut.h>
//#include <GLUT/glut.h>
#include <iostream>
#include <stdio.h>

//global struct to store the program state
struct ProgramState
{
  int cameraMode;

  int windowWidth;
  int windowHeight;
};
ProgramState state;

GLfloat position0[] = {50,-10,-10.0,1.0};

void DrawGrid(float width, int subdivision)
{
  //Draws a grid of GL_LINES as a reference point
  glDisable(GL_LIGHTING);
	
  glBegin(GL_LINES);
		
  for(int i = 0; i<=subdivision; i++)
    {	
      float stepsize = width/((float) (subdivision));
      float ratio = 1/((float) subdivision);
			
      glColor3f(1-i*ratio,i*ratio,0);
      glVertex3f(-width/2 + i*stepsize,0,width/2);
      glColor3f(i*ratio,1-i*ratio,0);
      glVertex3f(-width/2 + i*stepsize,0,-width/2);
		
      glColor3f(0,i*ratio,1-i*ratio);
      glVertex3f(width/2,0,-width/2 + i*stepsize);
      glColor3f(0,1-i*ratio,i*ratio);
      glVertex3f(-width/2,0,-width/2 + i*stepsize);
    }
  glEnd();

  glEnable(GL_LIGHTING);
}

void DrawCube(float x,float y, float z,float sx,float sy,float sz)
{
  //draw a parameterised rectangular prism
  glPushMatrix();
  glTranslatef(x,y,z);
  glScalef(sx,sy,sz);
  glutSolidCube(1.0);
  glPopMatrix();
}

void DrawPost(float x, float y, float z)
{
  //draws a chess rook at the specified coordinates
  //used to provide wayPoints in the scene

  glPushMatrix();
  glTranslatef(x,y,z);

  glColor3f(0.0,0.0,1.0);
	
  DrawCube(0,0,0,2,1.5,2);
  DrawCube(0,2.2,0,1.5,4,1.5);
  DrawCube(0,4.1,0,1.7,0.7,1.7);
	
  glPopMatrix();
}


void DisplayFunc()
{
  //clear screen
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
  //Set up Projection matrix
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  float aspect = (float) state.windowWidth / (float) state.windowHeight;
  gluPerspective(45,aspect,1,1000);
	
  //Set up Modelview matrix
  glMatrixMode( GL_MODELVIEW );

  // specify the light position in world coordinates, so that it does
  // not move with the camera
  glLightfv( GL_LIGHT0, GL_POSITION, position0 );
	
  //push, so our modeling transforms do not disrupt the Modelview Matrix
  //if we do not preserve the modelview matrix, we cannot do camera-centric
  //control.
  glPushMatrix();
	
  DrawPost(-50,0,-50);
  DrawPost(-50,0,0);
  DrawPost(-50,0,50);
  DrawPost(0,0,-50);
  DrawPost(0,0,50);
  DrawPost(50,0,-50);
  DrawPost(50,0,0);
  DrawPost(50,0,-50);
  DrawGrid(100,100);
	
  glPopMatrix();

  // draw the buffer to the screen
  glFlush();	glutSwapBuffers();

}

void ReshapeFunc(int w, int h)
{
  //saves the window width and height when the window changes shape
  //used to compute the aspect ratio when setting the projection matrix
  glViewport( 0,0,w,h );
  state.windowWidth = w;
  state.windowHeight = h;
  glutPostRedisplay();
}

void KeyboardFunc(unsigned char key, int x, int y)
{
  //printf("Key pressed: %c\n",key);
	
	
  //global commands
  switch (key) 
    {
    case '`':
      exit (0);
      break;
    case '1':
      printf("Camera Mode 1\n");
      state.cameraMode = 1;
      break;
    case '2':
      printf("Camera Mode 2\n");
      state.cameraMode = 2;
      break;
    case '3':
      printf("Camera Mode 3\n");
      state.cameraMode = 3;
      break;
    }
  glutPostRedisplay();
}

void init()
{
  //initialise GL state variables
  GLfloat lightAmbient[] = {.2,.2,.2,1};
  GLfloat lightDiffuse[] = { 1,1,1,1};
  GLfloat lightSpecular[] = { 1,1,1,1};
  GLfloat lightZero[] = {0,0,0,1};
	
  glShadeModel(GL_FLAT);
  glClearColor(0, 0, 0, 0.0); 	//setting bg color
  glEnable(GL_DEPTH_TEST);      //required for z-buffer
  glViewport(0, 0, state.windowWidth, state.windowWidth);

  // set light properties
  glLightModelfv( GL_LIGHT_MODEL_AMBIENT, lightZero);	// global ambient light
  glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);
  //glLightfv( GL_LIGHT0, GL_POSITION, position0 );
	
  //initialise lighting state
  glEnable( GL_LIGHTING );
  glEnable( GL_LIGHT0 );
  glEnable(GL_COLOR_MATERIAL);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_NORMALIZE); 
	
  //set some arbitrary initial camera position:
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
  gluLookAt(50,25,10,0,0,0,0,1,0);

}

int main(int argc, char** argv)
{
  //initialise program state
  state.cameraMode = 1;
  state.windowWidth = 640;
  state.windowHeight = 480;

  //setup window
  glutInit(&argc, argv);
  //RGB color, z-buffer, double drawing buffer
  glutInitDisplayMode (GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
  glutInitWindowPosition(100,100);
  glutInitWindowSize(state.windowWidth,state.windowHeight);
	
  glutCreateWindow("CPSC 314 project 2");
	
  //Set glut callback functions
  glutDisplayFunc(DisplayFunc);
  glutKeyboardFunc(KeyboardFunc);
  glutReshapeFunc(ReshapeFunc);

  init();

  glutMainLoop();

  return 0;
}
